#This software is a rerelease of LowessNorm.R module of Microarray Expression
#Potential Suite with alterations in the input and ouput format in order to
#improve transition between modules of eMESS.

#Based on the methodology described in:
#Normalization of spotted microarray data using the popular Lowess
#Normalization approach presented in Yang, HW et al, "Normalization for cDNA
#Microarray Data", SPIE BiOS 2001, San Jose, California, January 2001
#(http://stat-www.berkeley.edu/users/terry/zarray/Html/papersindex.html).

#No part of this text or software may be used or reproduced in any matter
#whatsoever without written permission, except in the case of brief quotations
#embodied in critical articles of or for reviews.  For information address
#Brian W. Kirk (bwkbem@yahoo.com).
#Software Copyright (C) October 2002  Brian W. Kirk
#Form/Text Copyright (C) October 2002  Brian W. Kirk
#Released for use with R v1.5.1 (C)

#Copyright (C) October 2002  Brian W. Kirk
#Released for R v1.5.1 (C)

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

###############################################################################
#                                FUNCTIONS                                    #
###############################################################################
library(MASS);

#FUNCTION THAT CALCULATES FOR A VECTOR OF SIGNALS A ROBUST SUMMARY USING THE
#HUBER M-ESTIMATOR AND THE MAD ESTIMATOR FOR SCALE.  RESULTS ARE COMBINED INTO
#A DATA.FRAME, WITH INCOMPATIBLE SETS (IE WITH ONLY 1 ENTRY) AMENDED TO THE END
#OF THE DATA.FRAME.

  fhuber.summarize <- function(SignalVec=vec, Filter=filter,
                               NameIdentifier=name, HuberConf=k, HuberTol=tol)
  {
   #Prime Header of Output Table
   huber.report.template <- c("ProbeID", "mu", "s", "probes");
   
   #Determine the Number of Probes Present for each Probeset 
   probes <- NameIdentifier[Filter];
   probesetf <- factor(probes);
   probesets <- levels(probesetf);

   probe.number <- NA;
   for (i in 1:length(probesets)) {
      temp  <- length(probes[as.logical(match(
                            probes, probesets[i], nomatch=0))]);
      probe.number <- c(probe.number, temp)  
   }
   probe.number <- probe.number[2:(length(probe.number))];
   
   probe.number.table <- data.frame(cbind(probesets, probe.number))

   #Select Porbesets With More Than 1 Probe For Huber Analysis
   huber.ok <- probesets[which(as.numeric(
                                     as.vector(probe.number.table[,2])) > 1)];

   #Apply Estimators To Qualified Probesets
   if (length(huber.ok) > 0) {
     for (i in (1:length(huber.ok))) {

       #Test That More Than Half of the Probes Are Not Identical
       test.probe <- SignalVec[as.logical(
                        match(NameIdentifier, huber.ok[i], nomatch=0)*Filter)];
     
       test.probef <- factor(test.probe);
       test.probes <- levels(test.probef);
       level.lengths <- tapply(test.probe, test.probef, length);

       continue <- TRUE;
       for (j in 1:length(test.probes)) {
         if ((length(test.probe) - level.lengths[j]) < 0.5*length(test.probe)){
           continue <- FALSE
         }
       }

       #Process Accordingly Based on the Previous Test
       if (continue) {
         xtemp.matrix <-  as.matrix(huber(test.probe, k=HuberConf,
                                          tol=HuberTol));
         huber.report.template <- cbind(huber.report.template,
                        c(as.character(huber.ok[i]),
                          as.numeric(xtemp.matrix[1,1]),
                          as.numeric(xtemp.matrix[2,1]),
                          as.numeric(as.vector(
                              probe.number.table[as.logical(match(
                              probesets, huber.ok[i], nomatch=0)),2]))));
       } else {
         mu <- median(test.probe);
         scale <- sd(test.probe);

         huber.report.template <- cbind(huber.report.template,
                        c(as.character(huber.ok[i]),
                          mu,
                          scale,
                          as.numeric(as.vector(
                              probe.number.table[as.logical(match(
                              probesets, huber.ok[i], nomatch=0)),2]))));
   
       } 
     }
   }
   
   #Organize Porbesets That Did Not Qualify For Estimator Analysis
   huber.lost <- probesets[which(as.numeric(
                                    as.vector(probe.number.table[,2])) <= 1)];
   if (length(huber.lost) > 0) {
     for (i in (1:length(huber.lost))) {
       huber.report.template <- cbind(huber.report.template,
                            c(as.character(huber.lost[i]),
                              as.numeric(SignalVec
                                          [as.logical(match(NameIdentifier,
                                           huber.lost[i], nomatch=0)
                                           * Filter)]), NA, 1))
     }
   }

   #Combine Both Probeset Types Into A Single Report
   huber.report.template2 <- t(huber.report.template);
   huber.report.template3 <-
                        huber.report.template2[2:(length(probesets)+1),];
   huber.sort.filter <- order(as.numeric(huber.report.template3[,4]),
                               decreasing=TRUE );
   return(data.frame(cbind(
                  ProbeID=
                    as.character(huber.report.template3[huber.sort.filter, 1]),
                  mu=
                    as.numeric(huber.report.template3[huber.sort.filter, 2]),
                  s=
                    as.numeric(huber.report.template3[huber.sort.filter, 3]),
                  probes=
                    as.numeric(
                    as.vector(huber.report.template3[huber.sort.filter, 4])))))
  }



###############################################################################
#                           CONSTANT DECLARATION                              #
###############################################################################

#CONSTANTS AND VARIABLES OF TEMPORARY USE BEGIN WITH "x", "y", or "z" AND ALL
#USER DEFINED FUNCTIONS BEGIN WITH "f" or "g" IN ORDER TO FACILITATE REMOVABLE
#FROM MEMORY.  WHEN USING VARIABLES WITH THESE RESERVED INTIAL LETTERS,
#NEAREST CLEANUP SHOULD BE IDENTIFIED TO INSURE PROPER PERMANENCY

                ############################################
                ######## DEFINE PERMANENT CONSTANTS ########
                ############################################

#Define Input and Output Files
SignalData <- paste(InputDirectory, InputFile, sep ="");
OutputPrefix <- paste(OutputDirectory, OutputFile, sep="");

#DEFINE GRAPHICS WINDOW SIZE
WinWidth <- 8.5;           
WinHeight <- 11;

#PARAMETERS FOR LOWESS
LowessWindow <- rep(LowessfParam, length(PinSet));


             #####################################################
             ##########  DEFINE TEMPORARY CONSTANTS  #############
             #####################################################

#DEFINE COLUMN POSITION BASED ON SIGNAL INPUT FILE
xBlockPos <- 1;
xColumnPos <- 2;
xRowPos <- 3;
xIDPos <- 4;
xMasterFilterPos <- 5;
xSampIntensityPos <- 6;     
xRefIntensityPos <- 7;      

###############################################################################
#                             VARIABLE DECLARATION                            #
###############################################################################

#CONSTANTS AND VARIABLES OF TEMPORARY USE BEGIN WITH "x", "y", or "z" AND ALL
#USER DEFINED FUNCTIONS BEGIN WITH "f" or "g" IN ORDER TO FACILITATE REMOVABLE
#FROM MEMORY.  WHEN USING VARIABLES WITH THESE RESERVED INTIAL LETTERS,
#NEAREST CLEANUP SHOULD BE IDENTIFIED TO INSURE PROPER PERMANCY
   
               ###################################################
               ######### CREATE VECTORS FROM SignalTable #########
               ###################################################

#Input Table of Intensity Signals 
xSignalTable <- data.frame(read.delim(SignalData, skip=HeaderLength));
OutputPrefix <- paste(OutputDirectory, OutputFile, sep="");

#SORT MGPR BY BLOCK, COLUMN AND THEN ROW ALL IN ASCENDING ORDER
zBlockVec <- as.numeric(as.vector(xSignalTable[, xBlockPos]));
zColumnVec <- as.numeric(as.vector(xSignalTable[, xColumnPos]));
zRowVec <- as.numeric(as.vector(xSignalTable[, xRowPos]));

zSortFilter <- order(zBlockVec, zRowVec, zColumnVec);

#Extract Pertinent Columns from Input Table and Assign Each Column to a
#Unique Vector in Ssorted Form
zBlockVec <- as.numeric(as.vector(zBlockVec[zSortFilter]));
zColumnVec <- as.numeric(as.vector(zColumnVec[zSortFilter]));
zRowVec <- as.numeric(as.vector(zRowVec[zSortFilter]));
zIDVec <- as.character(as.vector(xSignalTable[zSortFilter, xIDPos]));
zMasterFilter <- as.logical(as.vector(
                            xSignalTable[zSortFilter, xMasterFilterPos]));
zSampIntensityVec <- as.numeric(as.vector(
                             xSignalTable[zSortFilter, xSampIntensityPos]));
zRefIntensityVec <- as.numeric(as.vector(
                             xSignalTable[zSortFilter, xRefIntensityPos]));

#Remove temporary variables/constants
rm(list=ls(pat="^x"));


###############################################################################
#                               MAIN BODY                                     #
###############################################################################

         ################################################################
         ######      CALCULATE THE Log FOLD CHANGE VALUES          ######
         ######   (IE LOG BASE 2 (SampIntensity/RefIntensity))     ######
         ######          FOR ALL PROBES THAT PASS THE              ###### 
         ######                  zMasterFilter                     ######
         ################################################################

xNullVec <- rep(NA, length(zIDVec));

LogSampIntensityVec <- xNullVec;
LogSampIntensityVec[zMasterFilter] <- log(zSampIntensityVec[zMasterFilter], 2);

LogRefIntensityVec <- xNullVec;
LogRefIntensityVec[zMasterFilter] <- log(zRefIntensityVec[zMasterFilter], 2);

LogFoldChngVec <- xNullVec;
LogFoldChngVec[zMasterFilter] <- LogSampIntensityVec[zMasterFilter] -
                                      LogRefIntensityVec[zMasterFilter];


             #########################################################
             ####    NORMALIZE DATA BASED ON LOWESS SUBGRIDED     #### 
             #########################################################

Pinf <- factor(PinSet);
Pins <- levels(Pinf);

#Intialize output vector
LowessLogFoldChngVec <- rep(NA, length(zMasterFilter));

PinMList <- vector("list", length(Pins));
PinAList <- vector("list", length(Pins));
PinLowessMList <- vector("list", length(Pins));
LowessM <- vector("list", length(Pins));
LowessA <- vector("list", length(Pins));

for (i in 1:length(Pins)) {
  xBlockToPin <- BlockSet[as.logical(match(PinSet, Pins[i], nomatch=0))];
  xBlockFilter <- as.logical(match(zBlockVec, xBlockToPin, nomatch=0));
  xBlockMastFilter <- as.logical(xBlockFilter * zMasterFilter);
  
  PinAList[[i]] <- 0.5 * (LogSampIntensityVec[xBlockMastFilter]
                       + LogRefIntensityVec[xBlockMastFilter]);
  PinLogFold <- LogFoldChngVec[xBlockMastFilter];
  PinMList[[i]] <- PinLogFold;
  
  xNormList <- lowess(PinAList[[i]], PinMList[[i]], f=LowessWindow[i]);
  LowessA[[i]] <- xNormList$x;
  LowessM[[i]] <- xNormList$y;

  #Match LowessM Value to Corresponding PinLogFold Value and Correct
  LNLogFold <- 0;
  LowessFilter <- order(PinAList[[i]]);
  for (j in 1:length(LowessFilter)) {
    tempLNLogFold <- PinLogFold[j] - xNormList$y[as.logical(
                                         match(LowessFilter, j, nomatch=0))];
    LNLogFold <- c(LNLogFold, tempLNLogFold);
  }
                              
  PinLowessMList[[i]] <- LNLogFold[2:(length(LNLogFold))]
    
  #Map Normalized Log Fold Change Value to its Position Corresponding to the
  #Original Ordered Input Table (ie Signal TABLE)
  LowessLogFoldChngVec[xBlockMastFilter] <- PinLowessMList[[i]];
}

#Map Normalized Values to the Position Corresponding to the
#Original Ordered Input Table (ie Signal TABLE) 
LowessLogSampIntensityVec <- xNullVec;
LowessLogSampIntensityVec[zMasterFilter] <-
                                        LowessLogFoldChngVec[zMasterFilter] +
                                        LogRefIntensityVec[zMasterFilter];

LowessRMSIntensity <- xNullVec;
LowessRMSIntensity[zMasterFilter] <-
                         0.5 * (LowessLogSampIntensityVec[zMasterFilter]
                              + LogRefIntensityVec[zMasterFilter]);

RMSIntensity <- xNullVec;
RMSIntensity[zMasterFilter] <- 0.5 *
                                (LogSampIntensityVec[zMasterFilter] +
                                 LogRefIntensityVec[zMasterFilter]);


zProbeDataTable <- data.frame(cbind(
               Block=as.numeric(as.vector(zBlockVec)),
               Row=as.numeric(as.vector(zRowVec)),
               Column=as.numeric(as.vector(zColumnVec)),
               ProbeID=as.character(as.vector(zIDVec)),
               Filter=as.numeric(as.vector(zMasterFilter)),
               LNLogFoldChng=as.numeric(as.vector(LowessLogFoldChngVec)),
               LNLogGeoIntens=as.numeric(as.vector(LowessRMSIntensity)),
               LNLogSamp=as.numeric(as.vector(LowessLogSampIntensityVec)),
               LogFoldChng=as.numeric(as.vector(LogFoldChngVec)),
               LogGeomIntens=as.numeric(as.vector(RMSIntensity)),
               LogSamp=as.numeric(as.vector(LogSampIntensityVec)),
               LogRef=as.numeric(as.vector(LogRefIntensityVec))));



#Create Probeset Based Table Using Robust Summary Statistics for Location
#and Scale

zMHuberReport <- fhuber.summarize(LowessLogFoldChngVec, zMasterFilter,
                                zIDVec, HuberConf, HuberTol);

zAHuberReport <- fhuber.summarize(RMSIntensity, zMasterFilter,
                                zIDVec, HuberConf, HuberTol);


zProbesetDataTable <- data.frame(cbind(
                          ProbeID=as.vector(zMHuberReport[,1]),
                          MuFold=as.numeric(as.vector(zMHuberReport[,2])),
                          MuAmp=as.numeric(as.vector(zAHuberReport[,2])),
                          NumberOfProbes=as.vector(zAHuberReport[,4]),
                          SFold=as.vector(zMHuberReport[,3]),
                          SAmp=as.vector(zAHuberReport[,3])));

#Remove temporary variables/constants
rm(list=ls(pat="^x"));
rm(list=ls(pat="^f"));

##############################################################################
#                              OUTPUT RESULTS                                #
##############################################################################

                      ################################
                      #######   TABLES OUTPUT   ######
                      ################################
                              
#zProbeDataTable
FilterHeader1 <- paste(SignalData);
FilterHeader2 <- date();
FilterHeader3 <- c("PinSet = ", paste(as.character(PinSet), sep=""));
FilterHeader4 <- c("BlockSet = ", paste(as.character(BlockSet), sep=""));     
FilterHeader5 <- paste("LowessfParam = ", as.character(LowessfParam),
                         sep="");          
FilterHeader6 <- c(paste("HuberConf = ", as.character(HuberConf), sep=""),
                   paste("HuberTol = ", as.character(HuberTol), sep=""));
FilterHeaderFill <- rep(" ", 3);
FilterHeaderTable <-  c("Block", "Row", "Column", "ProbeID", "Filter",
                        "LNLogFoldChng", "LNLogGeoIntens", "LNLogSamp",
                        "LogFoldChng", "LogGeomIntens", "LogSamp", "LogRef");
                              
cat(FilterHeader1, file=paste(OutputPrefix, ".LowProbe", sep=""),
      sep="\n");
cat(FilterHeader2, file=paste(OutputPrefix, ".LowProbe", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader3, file=paste(OutputPrefix, ".LowProbe", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".LowProbe", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader4, file=paste(OutputPrefix, ".LowProbe", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".LowProbe", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader5, file=paste(OutputPrefix, ".LowProbe", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".LowProbe", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader6, file=paste(OutputPrefix, ".LowProbe", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".LowProbe", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeaderFill, file=paste(OutputPrefix, ".LowProbe", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeaderTable, file=paste(OutputPrefix, ".LowProbe", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".LowProbe", sep=""),
      sep="\n", append=TRUE);
  
write.table(zProbeDataTable, file=paste(OutputPrefix, ".LowProbe",
                 sep=""), quote=FALSE, sep="\t", col.names=FALSE,
                 row.names=FALSE, append=TRUE);


#zProbesetDataTable
FilterHeader1 <- paste(SignalData);
FilterHeader2 <- date();
FilterHeader3 <- c("PinSet = ", as.character(PinSet), sep="");
FilterHeader4 <- c("BlockSet = ", as.character(BlockSet), sep="");     
FilterHeader5 <- paste("LowessfParam = ", as.character(LowessfParam),
                         sep="");          
FilterHeader6 <- c(paste("HuberConf = ", as.character(HuberConf), sep=""),
                   paste("HuberTol = ", as.character(HuberTol), sep=""));
FilterHeaderFill <- rep(" ", 3);
FilterHeaderTable <-  c("ID", "MuFold", "MuAmp", "NumberOfProbes", "SFold",
                        "SAmp");
                              
cat(FilterHeader1, file=paste(OutputPrefix, ".LowPset", sep=""),
      sep="\n");
cat(FilterHeader2, file=paste(OutputPrefix, ".LowPset", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader3, file=paste(OutputPrefix, ".LowPset", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".LowPset", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader4, file=paste(OutputPrefix, ".LowPset", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".LowPset", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader5, file=paste(OutputPrefix, ".LowPset", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".LowPset", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader6, file=paste(OutputPrefix, ".LowPset", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".LowPset", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeaderFill, file=paste(OutputPrefix, ".LowPset", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeaderTable, file=paste(OutputPrefix, ".LowPset", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".LowPset", sep=""),
      sep="\n", append=TRUE);
  
write.table(zProbesetDataTable, file=paste(OutputPrefix, ".LowPset",
                 sep=""), quote=FALSE, sep="\t", col.names=FALSE,
                 row.names=FALSE, append=TRUE);




                      ################################
                      #######  GRAPHICS OUTPUT  ######
                      ################################     

if (PlotResults) {

  #Set Pin colors for M vs A PLOTS
  if (round(length(Pins)/7) != 0) {
    colors <- rep(1:7, round(length(Pins)/7));
  } else {
    colors <- 1:length(Pins);
  }

  ####  M v A Raw Data
  postscript(file=paste(OutputPrefix, "MvAraw.eps", sep=""), horizontal=TRUE,
             onefile=TRUE);

  chh <-par()$cxy[2];
  chw <-par()$cxy[2];
  par(mar=c(4,4,3,2), omi=c(0.5,0.5,0.5,0.5));
  
  y0 <- LogFoldChngVec[zMasterFilter];
  x0 <- RMSIntensity[zMasterFilter];
  
  plot(y0 ~ x0, type="n",  main="Raw Data",
     xlab="A", ylab = "M",
     cex= 1.0, xlim=c(0, max(x0)), ylim=c(min(y0), max(y0)),
     pch=16);
  abline(h=0);
  for (i in 1:length(Pins)) {
    text(PinAList[[i]], PinMList[[i]], labels=i, cex=0.5, col=colors[i]);
  }
  dev.off()

  ### M v A Lowess Normalized Data
  postscript(file=paste(OutputPrefix, "MvAnorm.eps", sep=""), horizontal=TRUE,
             onefile=TRUE);

  chh <-par()$cxy[2];
  chw <-par()$cxy[2];
  par(mar=c(4,4,3,2), omi=c(0.5,0.5,0.5,0.5));

  y0 <- LowessLogFoldChngVec[zMasterFilter];
  x0 <- RMSIntensity[zMasterFilter];
  
  plot(y0 ~ x0, type="n",  main="Lowess Pin Normalization", 
     xlab="A", ylab = "M",
     cex= 1.0, xlim=c(0, max(x0)), ylim=c(min(y0), max(y0)),
     pch=16);
  abline(h=0);
  for (i in 1:length(Pins)) {
    text(PinAList[[i]], PinLowessMList[[i]], labels=i, cex=0.5, col=colors[i]);
  }
  dev.off()
  
  
  #M v A Pins Combined
  postscript(file=paste(OutputPrefix, "PinsComb.eps", sep=""),
             horizontal=FALSE, onefile=TRUE);

  chh <-par()$cxy[2];
  chw <-par()$cxy[2];
  par(mar=c(4,4,3,2), omi=c(0.5,0.5,0.5,0.5));

  par(mfrow=c(2,1));

  y0 <- LogFoldChngVec[zMasterFilter];
  x0 <- RMSIntensity[zMasterFilter];
  
  plot(y0 ~ x0, type="p",  main="Lowess Pin Normalization",
     xlab="A", ylab = "M",
     cex= 0.1, xlim=c(0, max(x0)), ylim=c(min(y0), max(y0)),
     pch=16);
  abline(h=0);
  for (i in 1:length(Pins)) {
    lines(LowessA[[i]], LowessM[[i]], col=colors[i], lty=i);
  }

  y0 <- PinMList[[i]];
  x0 <- PinAList[[i]];
  
  plot(y0 ~ x0, type="n",  main="Lowess Pin Normalization",
     xlab="A", ylab = "M",
     cex= 0.1, xlim=c(0, max(x0)), ylim=c(min(y0), max(y0)),
     pch=16);
  abline(h=0);
  for (i in 1:length(Pins)) {
    lines(LowessA[[i]], LowessM[[i]], col=colors[i], lty=1, lwd=2);
  }
  dev.off()

  #M v A Individual Pins
  if (length(Pins) >= 8 ){
    for (i in 1:trunc(length(Pins)/8)) {
      postscript(file=paste(OutputPrefix, "PinInd", i, ".eps", sep=""),
                 horizontal=FALSE, onefile=TRUE);

      chh <-par()$cxy[2];
      chw <-par()$cxy[2];
  
      par(mar=c(3,2,2,2), omi=c(0.5,0.5,0.5,0.5));
      par(mfrow=c(4,2))
      for (j in 1:8) {
        y0 <- PinMList[[(i-1)*8 + j]];
        x0 <- PinAList[[(i-1)*8 + j]];
  
        plot(y0 ~ x0, type="p", main=paste("Pin", (i-1)*8 + j, sep=" "),
          xlab="A", ylab = "M", cex= 0.1,
          xlim=c(0, max(x0)), ylim=c(min(y0), max(y0)),
          pch=16);
        abline(h=0);
        lines(LowessA[[(i-1)*8 + j]], LowessM[[(i-1)*8 + j]],
            col=colors[j], lty=1, lwd=2);
      }
      dev.off()
    }
  } else {i <- 0}
  
  if (length(Pins) %% 8  >= 4 ){
    postscript(file=paste(OutputPrefix, "PinInd", i+1, ".eps", sep=""),
               horizontal=FALSE, onefile=TRUE);

    chh <-par()$cxy[2];
    chw <-par()$cxy[2];
 
    par(mar=c(3,2,2,2), omi=c(0.5,0.5,0.5,0.5));
    par(mfrow=c(4,1))
    for (j in 1:4) {
      y0 <- PinMList[[i*8 + j]];
      x0 <- PinAList[[i*8 + j]];
  
      plot(y0 ~ x0, type="p", main=paste("Pin", i*8 + j, sep=" "),
        xlab="A", ylab = "M", cex= 0.1,
        xlim=c(0, max(x0)), ylim=c(min(y0), max(y0)),
        pch=16);
      abline(h=0);
      lines(LowessA[[i*8 + j]], LowessM[[i*8 + j]],
            col=colors[j], lty=1, lwd=2);
    }
    k <- 4;
    dev.off()
  } else {k <- 0}
    
  if (length(Pins) %% 4 != 0) {
    postscript(file=paste(OutputPrefixPlots, "PinInd", i+2, ".eps", sep=""),
               horizontal=FALSE, onefile=TRUE);

    chh <-par()$cxy[2];
    chw <-par()$cxy[2];
  
    par(mar=c(3,2,2,2), omi=c(0.5,0.5,0.5,0.5));
    par(mfrow=c(length(Pins) %% 4, 1))
    for (j in 1:(length(Pins) %% 4)) {
      y0 <- PinMList[[i*8 + k + j]];
      x0 <- PinAList[[i*8 + k + j]];
  
      plot(y0 ~ x0, type="p", main=paste("Pin", i*8 + k + j, sep=" "),
        xlab="A", ylab = "M", cex= 0.1,
        xlim=c(0, max(x0)), ylim=c(min(y0), max(y0)),
        pch=16);
      abline(h=0);
      lines(LowessA[[i*8 + j]], LowessM[[i*8 + j]],
            col=colors[j], lty=1, lwd=2);
    }
    dev.off()
  }


}

rm(list=ls());
